# coding: utf-8
"""
    LAMP Platform

    The LAMP Platform API.

    The version of the OpenAPI document: 1.0.0
    Contact: team@digitalpsych.org
"""

from __future__ import absolute_import

import json
import atexit
import mimetypes
from multiprocessing.pool import ThreadPool
import os

# python 2 and python 3 compatibility library
import six
from six.moves.urllib.parse import quote

from LAMP import rest
from LAMP.configuration import Configuration
from LAMP.exceptions import ApiValueError
from LAMP.model_utils import (
    ModelNormal,
    ModelSimple,
    ModelComposed,
    date,
    datetime,
    deserialize_file,
    file_type,
    model_to_dict,
    str,
    validate_and_convert_types
)


class ApiClient(object):
    """Generic API client for OpenAPI client library builds.

    OpenAPI generic API client. This client handles the client-
    server communication, and is invariant across implementations. Specifics of
    the methods and models for each application are generated from the OpenAPI
    templates.

    NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech
    Do not edit the class manually.

    :param configuration: .Configuration object for this client
    :param header_name: a header to pass when making calls to the API.
    :param header_value: a header value to pass when making calls to
        the API.
    :param cookie: a cookie to include in the header when making calls
        to the API
    :param pool_threads: The number of threads to use for async requests
        to the API. More threads means more concurrent API requests.
    """

    # six.binary_type python2=str, python3=bytes
    # six.text_type python2=unicode, python3=str
    PRIMITIVE_TYPES = (
        (float, bool, six.binary_type, six.text_type) + six.integer_types
    )
    _pool = None

    def __init__(self, configuration=None, header_name=None, header_value=None,
                 cookie=None, pool_threads=1):
        if configuration is None:
            configuration = Configuration()
        self.configuration = configuration
        self.pool_threads = pool_threads

        self.rest_client = rest.RESTClientObject(configuration)
        self.default_headers = {}
        if header_name is not None:
            self.default_headers[header_name] = header_value
        self.cookie = cookie
        # Set default User-Agent.
        self.user_agent = 'OpenAPI-Generator/1.0.0/python'

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def close(self):
        if self._pool:
            self._pool.close()
            self._pool.join()
            self._pool = None
            if hasattr(atexit, 'unregister'):
                atexit.unregister(self.close)

    @property
    def pool(self):
        """Create thread pool on first request
         avoids instantiating unused threadpool for blocking clients.
        """
        if self._pool is None:
            atexit.register(self.close)
            self._pool = ThreadPool(self.pool_threads)
        return self._pool

    @property
    def user_agent(self):
        """User agent for this API client"""
        return self.default_headers['User-Agent']

    @user_agent.setter
    def user_agent(self, value):
        self.default_headers['User-Agent'] = value

    def set_default_header(self, header_name, header_value):
        self.default_headers[header_name] = header_value

    def __call_api(
            self, resource_path, method, path_params=None,
            query_params=None, header_params=None, body=None, post_params=None,
            files=None, response_type=None, auth_settings=None,
            _return_http_data_only=None, collection_formats=None,
            _preload_content=True, _request_timeout=None, _host=None,
            _check_type=None):

        config = self.configuration

        # header parameters
        header_params = header_params or {}
        header_params.update(self.default_headers)
        if self.cookie:
            header_params['Cookie'] = self.cookie
        if header_params:
            header_params = self.sanitize_for_serialization(header_params)
            header_params = dict(self.parameters_to_tuples(header_params,
                                                           collection_formats))

        # path parameters
        if path_params:
            path_params = self.sanitize_for_serialization(path_params)
            path_params = self.parameters_to_tuples(path_params,
                                                    collection_formats)
            for k, v in path_params:
                # specified safe chars, encode everything
                resource_path = resource_path.replace(
                    '{%s}' % k,
                    quote(str(v), safe=config.safe_chars_for_path_param)
                )

        # query parameters
        if query_params:
            query_params = self.sanitize_for_serialization(query_params)
            query_params = self.parameters_to_tuples(query_params,
                                                     collection_formats)

        # post parameters
        if post_params or files:
            post_params = post_params if post_params else []
            post_params = self.sanitize_for_serialization(post_params)
            post_params = self.parameters_to_tuples(post_params,
                                                    collection_formats)
            post_params.extend(self.files_parameters(files))

        # body
        if body:
            body = self.sanitize_for_serialization(body)

        # auth setting
        self.update_params_for_auth(header_params, query_params,
                                    auth_settings, resource_path, method, body)

        # request url
        if _host is None:
            url = self.configuration.host + resource_path
        else:
            # use server/host defined in path or operation instead
            url = _host + resource_path

        # perform request and return response
        response_data = self.request(
            method, url, query_params=query_params, headers=header_params,
            post_params=post_params, body=body,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout)

        self.last_response = response_data

        return_data = response_data
        if _preload_content:
            # deserialize response data
            if response_type:
                return_data = self.deserialize(
                    response_data,
                    response_type,
                    _check_type
                )
            else:
                return_data = None

        if _return_http_data_only:
            return (return_data)
        else:
            return (return_data, response_data.status,
                    response_data.getheaders())

    def sanitize_for_serialization(self, obj):
        """Builds a JSON POST object.

        If obj is None, return None.
        If obj is str, int, long, float, bool, return directly.
        If obj is datetime.datetime, datetime.date
            convert to string in iso8601 format.
        If obj is list, sanitize each element in the list.
        If obj is dict, return the dict.
        If obj is OpenAPI model, return the properties dict.

        :param obj: The data to serialize.
        :return: The serialized form of data.
        """
        if obj is None:
            return None
        elif isinstance(obj, self.PRIMITIVE_TYPES):
            return obj
        elif isinstance(obj, list):
            return [self.sanitize_for_serialization(sub_obj)
                    for sub_obj in obj]
        elif isinstance(obj, tuple):
            return tuple(self.sanitize_for_serialization(sub_obj)
                         for sub_obj in obj)
        elif isinstance(obj, (datetime, date)):
            return obj.isoformat()

        if isinstance(obj, dict):
            obj_dict = obj
        elif isinstance(obj, ModelNormal) or isinstance(obj, ModelComposed):
            # Convert model obj to dict
            # Convert attribute name to json key in
            # model definition for request
            obj_dict = model_to_dict(obj, serialize=True)
        elif isinstance(obj, ModelSimple):
            return self.sanitize_for_serialization(obj.value)

        return {key: self.sanitize_for_serialization(val)
                for key, val in six.iteritems(obj_dict)}

    def deserialize(self, response, response_type, _check_type):
        """Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: For the response, a tuple containing:
            valid classes
            a list containing valid classes (for list schemas)
            a dict containing a tuple of valid classes as the value
            Example values:
            (str,)
            (Pet,)
            (float, none_type)
            ([int, none_type],)
            ({str: (bool, str, int, float, date, datetime, str, none_type)},)
        :param _check_type: boolean, whether to check the types of the data
            received from the server

        :return: deserialized object.
        """
        # handle file downloading
        # save response body into a tmp file and return the instance
        if response_type == (file_type,):
            content_disposition = response.getheader("Content-Disposition")
            return deserialize_file(response.data, self.configuration,
                                    content_disposition=content_disposition)

        # fetch data from response object
        try:
            received_data = json.loads(response.data)
        except ValueError:
            received_data = response.data

        # store our data under the key of 'received_data' so users have some
        # context if they are deserializing a string and the data type is wrong
        deserialized_data = validate_and_convert_types(
            received_data,
            response_type,
            ['received_data'],
            True,
            _check_type,
            configuration=self.configuration
        )
        return deserialized_data

    def call_api(self, resource_path, method,
                 path_params=None, query_params=None, header_params=None,
                 body=None, post_params=None, files=None,
                 response_type=None, auth_settings=None, async_req=None,
                 _return_http_data_only=None, collection_formats=None,
                 _preload_content=True, _request_timeout=None, _host=None,
                 _check_type=None):
        """Makes the HTTP request (synchronous) and returns deserialized data.

        To make an async_req request, set the async_req parameter.

        :param resource_path: Path to method endpoint.
        :param method: Method to call.
        :param path_params: Path parameters in the url.
        :param query_params: Query parameters in the url.
        :param header_params: Header parameters to be
            placed in the request header.
        :param body: Request body.
        :param post_params dict: Request post form parameters,
            for `application/x-www-form-urlencoded`, `multipart/form-data`.
        :param auth_settings list: Auth Settings names for the request.
        :param response_type: For the response, a tuple containing:
            valid classes
            a list containing valid classes (for list schemas)
            a dict containing a tuple of valid classes as the value
            Example values:
            (str,)
            (Pet,)
            (float, none_type)
            ([int, none_type],)
            ({str: (bool, str, int, float, date, datetime, str, none_type)},)
        :param files dict: key -> field name, value -> a list of open file
            objects for `multipart/form-data`.
        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param collection_formats: dict of collection formats for path, query,
            header, and post parameters.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _check_type: boolean describing if the data back from the server
            should have its type checked.
        :return:
            If async_req parameter is True,
            the request will be called asynchronously.
            The method will return the request thread.
            If parameter async_req is False or missing,
            then the method will return the response directly.
        """
        if not async_req:
            return self.__call_api(resource_path, method,
                                   path_params, query_params, header_params,
                                   body, post_params, files,
                                   response_type, auth_settings,
                                   _return_http_data_only, collection_formats,
                                   _preload_content, _request_timeout, _host,
                                   _check_type)

        return self.pool.apply_async(self.__call_api, (resource_path,
                                                       method, path_params,
                                                       query_params,
                                                       header_params, body,
                                                       post_params, files,
                                                       response_type,
                                                       auth_settings,
                                                       _return_http_data_only,
                                                       collection_formats,
                                                       _preload_content,
                                                       _request_timeout,
                                                       _host, _check_type))

    def request(self, method, url, query_params=None, headers=None,
                post_params=None, body=None, _preload_content=True,
                _request_timeout=None):
        """Makes the HTTP request using RESTClient."""
        if method == "GET":
            return self.rest_client.GET(url,
                                        query_params=query_params,
                                        _preload_content=_preload_content,
                                        _request_timeout=_request_timeout,
                                        headers=headers)
        elif method == "HEAD":
            return self.rest_client.HEAD(url,
                                         query_params=query_params,
                                         _preload_content=_preload_content,
                                         _request_timeout=_request_timeout,
                                         headers=headers)
        elif method == "OPTIONS":
            return self.rest_client.OPTIONS(url,
                                            query_params=query_params,
                                            headers=headers,
                                            post_params=post_params,
                                            _preload_content=_preload_content,
                                            _request_timeout=_request_timeout,
                                            body=body)
        elif method == "POST":
            return self.rest_client.POST(url,
                                         query_params=query_params,
                                         headers=headers,
                                         post_params=post_params,
                                         _preload_content=_preload_content,
                                         _request_timeout=_request_timeout,
                                         body=body)
        elif method == "PUT":
            return self.rest_client.PUT(url,
                                        query_params=query_params,
                                        headers=headers,
                                        post_params=post_params,
                                        _preload_content=_preload_content,
                                        _request_timeout=_request_timeout,
                                        body=body)
        elif method == "PATCH":
            return self.rest_client.PATCH(url,
                                          query_params=query_params,
                                          headers=headers,
                                          post_params=post_params,
                                          _preload_content=_preload_content,
                                          _request_timeout=_request_timeout,
                                          body=body)
        elif method == "DELETE":
            return self.rest_client.DELETE(url,
                                           query_params=query_params,
                                           headers=headers,
                                           _preload_content=_preload_content,
                                           _request_timeout=_request_timeout,
                                           body=body)
        else:
            raise ApiValueError(
                "http method must be `GET`, `HEAD`, `OPTIONS`,"
                " `POST`, `PATCH`, `PUT` or `DELETE`."
            )

    def parameters_to_tuples(self, params, collection_formats):
        """Get parameters as list of tuples, formatting collections.

        :param params: Parameters as dict or list of two-tuples
        :param dict collection_formats: Parameter collection formats
        :return: Parameters as list of tuples, collections formatted
        """
        new_params = []
        if collection_formats is None:
            collection_formats = {}
        for k, v in six.iteritems(params) if isinstance(params, dict) else params:
            if k in collection_formats:
                collection_format = collection_formats[k]
                if collection_format == 'multi':
                    new_params.extend((k, value) for value in v)
                else:
                    if collection_format == 'ssv':
                        delimiter = ' '
                    elif collection_format == 'tsv':
                        delimiter = '\t'
                    elif collection_format == 'pipes':
                        delimiter = '|'
                    else:  # csv is the default
                        delimiter = ','
                    new_params.append(
                        (k, delimiter.join(str(value) for value in v)))
            else:
                new_params.append((k, v))
        return new_params

    def files_parameters(self, files=None):
        """Builds form parameters.

        :param files: None or a dict with key=param_name and
            value is a list of open file objects
        :return: List of tuples of form parameters with file data
        """
        if files is None:
            return []

        params = []
        for param_name, file_instances in six.iteritems(files):
            if file_instances is None:
                # if the file field is nullable, skip None values
                continue
            for file_instance in file_instances:
                if file_instance is None:
                    # if the file field is nullable, skip None values
                    continue
                if file_instance.closed is True:
                    raise ApiValueError(
                        "Cannot read a closed file. The passed in file_type "
                        "for %s must be open." % param_name
                    )
                filename = os.path.basename(file_instance.name)
                filedata = file_instance.read()
                mimetype = (mimetypes.guess_type(filename)[0] or
                            'application/octet-stream')
                params.append(
                    tuple([param_name, tuple([filename, filedata, mimetype])]))
                file_instance.close()

        return params

    def select_header_accept(self, accepts):
        """Returns `Accept` based on an array of accepts provided.

        :param accepts: List of headers.
        :return: Accept (e.g. application/json).
        """
        if not accepts:
            return

        accepts = [x.lower() for x in accepts]

        if 'application/json' in accepts:
            return 'application/json'
        else:
            return ', '.join(accepts)

    def select_header_content_type(self, content_types):
        """Returns `Content-Type` based on an array of content_types provided.

        :param content_types: List of content-types.
        :return: Content-Type (e.g. application/json).
        """
        if not content_types:
            return 'application/json'

        content_types = [x.lower() for x in content_types]

        if 'application/json' in content_types or '*/*' in content_types:
            return 'application/json'
        else:
            return content_types[0]

    def update_params_for_auth(self, headers, querys, auth_settings,
                               resource_path, method, body):
        """Updates header and query params based on authentication setting.

        :param headers: Header parameters dict to be updated.
        :param querys: Query parameters tuple list to be updated.
        :param auth_settings: Authentication setting identifiers list.
        :resource_path: A string representation of the HTTP request resource path.
        :method: A string representation of the HTTP request method.
        :body: A object representing the body of the HTTP request.
            The object type is the return value of sanitize_for_serialization().
        """
        if not auth_settings:
            return

        for auth in auth_settings:
            auth_setting = self.configuration.auth_settings().get(auth)
            if auth_setting:
                if auth_setting['in'] == 'cookie':
                    headers['Cookie'] = auth_setting['value']
                elif auth_setting['in'] == 'header':
                    if auth_setting['type'] != 'http-signature':
                        headers[auth_setting['key']] = auth_setting['value']
                elif auth_setting['in'] == 'query':
                    querys.append((auth_setting['key'], auth_setting['value']))
                else:
                    raise ApiValueError(
                        'Authentication token must be in `query` or `header`'
                    )



import pandas as pd 
import numpy as np
import datetime
import os
import math
import LAMP
import itertools
from functools import reduce
#import

class ParticipantExt():
    """
    Create subject dataframe


    """
    def __init__(self, 
                 id, 
                 domains=None, 
                 age=None, 
                 race=None, 
                 sex=None, 
                 beiwe_id=None, 
                 beiwe_filepath=None, 
                 beta_values_filepath=None,
                 df_props={}):

        self.id = id
        self.domains = domains
        self.beiwe_id = beiwe_id
        self.beiwe_filepath = beiwe_filepath
        self.beta_values_filepath = beta_values_filepath
        self.age = age
        self.race = race
        self.sex = sex

        self.df = self.create_subject_df(**df_props)

        self.impute_status = False
        self.bin_status = False
        self.normalize_status = False	

    @property
    def id(self):
        return self._id

    @property
    def age(self):
        return self._age

    @property
    def race(self):
        return self._race

    @property
    def sex(self):
        return self._sex

    @property
    def beiwe_id(self):
        return self._beiwe_id

    @property
    def beiwe_filepath(self):
        return self._beiwe_filepath
    
    @property
    def beta_values_filepath(self):
        return self._beta_values_filepath

    @id.setter
    def id(self, value):
        self._id = value

    @age.setter
    def age(self, value):
        self._age = value

    @race.setter
    def race(self, value):
        self._race = value

    @sex.setter
    def sex(self, value):
        self._sex = value

    @beiwe_id.setter
    def beiwe_id(self, value):
        self._beiwe_id = value

    @beiwe_filepath.setter
    def beiwe_filepath(self, value):
        self._beiwe_filepath = value
        
    @beta_values_filepath.setter
    def beta_values_filepath(self, value):
        self._beta_values_filepath = value

    def reset(self):
        """
        Resets the subject's df to be the original version
        """
        self.df = self.create_subject_df()
        self.impute_status, self.bin_status, self.normalize_status = False, False, False

    def domain_check(self, domains):
        """
        If domains is passed in, just return it.

        Else, see if domains is set as object attribute
        """
        if domains is None:
            if not hasattr(self, 'domains'):
                raise AttributeError('Domains were not set for cohort and were not provided.')
            domains = self.domains
        return domains


    def survey_results(self, participant=None, question_categories=None):
        """
        Get survey events for participant

        :param participant (str): the LAMP ID for participant. If not provided, then take subjects
        :param question_categories (bool): indicates whether to use custom question mappings as defined in params file
        """
        def survey_results_with_custom_categories(participant_results, question_categories):
            """
            """

            participant_surveys = {}
            for result in participant_results:
                survey_time = result['timestamp']

                #Parse survey event with defined questions
                survey_result = {}
                for event in result['temporal_slices']:
                    question = event['item']                    
                    if question in question_categories: #Check if question in one of the categories
                        score = int(event['value'])
                        if question_categories[question]['category']['reverse_scoring']:
                            score = 3 - score 
                        category = question_categories[question]['category']
                        if category in survey_result: survey_result[category].append(score) 
                        else: survey_result[category] = [score]

                #Take mean for each cat
                for cat in survey_result:
                    survey_result[cat] = float(sum(survey_result[cat])/len(survey_result[cat]))
                
                for category in survey_result: #Add to master dictionary
                    if category not in participant_surveys: participant_surveys[category] = [(survey_result[category], survey_time)]
                    else: participant_surveys[category].append((survey_result[category], survey_time))

            return participant_surveys


        if participant is None:
            participant = self.id
            

        participant_activities = LAMP.Activity.all_by_participant(participant)['data']
        participant_activities_surveys = [activity for activity in participant_activities if activity['spec'] == 'lamp.survey']
        participant_activities_surveys_ids = [survey['id'] for survey in participant_activities_surveys]        
        
        participant_results = [result for result in LAMP.ActivityEvent.all_by_participant(participant)['data'] if result['activity'] in participant_activities_surveys_ids and len(result['temporal_slices']) > 0]

        #Perform different parsing if user-defined question categories
        if question_categories:
            return survey_results_with_custom_categories(participant_results, question_categories)

        participant_surveys = {} #maps survey_type to occurence of scores 
        for result in participant_results:

            #Check if it's a survey event
            if result['activity'] not in participant_activities_surveys_ids or len(result['temporal_slices']) == 0: continue
            activity = LAMP.Activity.view(result['activity'])['data'][0]
            
            #Check to see if all the event values are numerical
            try: [float(event['value']) for event in result['temporal_slices']]
            except: continue
                
            survey_time = result['timestamp']
            survey_score = sum([float(event['value']) for event in result['temporal_slices']]) / len(result['temporal_slices'])
            
            #Add result
            if activity['name'] not in participant_surveys:
                participant_surveys[activity['name']] = []
            participant_surveys[activity['name']].append((survey_score, survey_time))
          
        #Sort surveys by time
        for activity_category in participant_surveys:
            participant_surveys[activity_category] = sorted(participant_surveys[activity_category], key=lambda x: x[1])

        return participant_surveys

    def create_subject_df(self, days_cap=120, day_first=None, day_last=None, resolution='day', start_monday=False, start_morning=True, time_centered=False, question_categories=None):
        """
        Create subject dataframe

        :param day_first (datetime.Date)
        """

        FIFTEEN_MIN_PER_UNIT = {'15 min': 1, 'day': 4*24, 'week': 4*24*7, 'month': 4*24*30}
        UNITS_PER_DAY = {'15 min': 4*24, 'day': 1, 'week': 1/7, 'month': 1/30}

        def parse_surveys(days_cap=120, day_first=None, day_last=None, resolution='day', start_monday=False, start_morning=True, time_centered=False, question_categories=None):            
            subject_surveys = self.survey_results(question_categories=question_categories)
            if len(subject_surveys) == 0:
                return None
            
            #Find the first, last date
            if day_first is None: day_first = datetime.datetime.utcfromtimestamp(sorted([subject_surveys[dom][0][1]/1000 for dom in subject_surveys])[0])
            else: day_first = datetime.datetime.combine(day_first, datetime.time.min) #convert to datetime

            if day_last is None: day_last = datetime.datetime.utcfromtimestamp(sorted([subject_surveys[dom][-1][1]/1000 for dom in subject_surveys])[-1])
            else: day_first = datetime.datetime.combine(day_first, datetime.time.min) #convert to datetime

            #Clip days based on morning and weekday parameters
            if start_monday:
                if day_first.weekday() > 0: 
                    day_first += datetime.timedelta(days = - day_first.weekday())

            if start_morning: 
                day_first, day_last = day_first.replace(hour=9, minute=0, second=0), day_last.replace(hour=9, minute=0, second=0)
            days_elapsed = (day_last - day_first).days 
            date_list = [day_first + datetime.timedelta(minutes=15*FIFTEEN_MIN_PER_UNIT[resolution]*x) for x in range(0, math.ceil(min(days_elapsed, days_cap) * UNITS_PER_DAY[resolution]))]

            #Create dateframe for the number of time units that have data; limited by days; cap at 'days_cap' if this number is large
            df = pd.DataFrame({'Date': date_list, 'id':self.id})
            
            if self.domains is None: domains = [dom for dom in subject_surveys] #if not set, just use cats from surveys
            else: domains = self.domains
            for dom in domains: 
                df[dom] = np.nan

            #Parse surveys
            for dom in subject_surveys:
                if dom not in domains and domains is not None:
                    continue

                #Based on resolution, match each survey event to its closest date
                dates = [datetime.datetime.utcfromtimestamp(event_time/1000) for _, event_time in subject_surveys[dom] if day_first <= datetime.datetime.utcfromtimestamp(event_time/1000) <= day_last] 

                #Choose closest date if "time centered"; else, choose preceding date
                if time_centered: rounded_dates = [df.loc[df.index[(date - df['Date']).abs().sort_values().index[0]], 'Date'] for date in dates]
                else: rounded_dates = [df.loc[df.index[(date - df['Date'])[(date - df['Date']) >= datetime.timedelta(0)].sort_values().index[0]], 'Date'] for date in dates]

                results = [event_val for event_val, event_time in subject_surveys[dom] if day_first <= datetime.datetime.utcfromtimestamp(event_time/1000) <= day_last]
                dom_results = pd.DataFrame({'Date':dates, 'Rounded Date':rounded_dates, 'Result':results})
                for date, date_df in dom_results.groupby('Rounded Date'):                    
                    df.loc[df['Date'] == date.to_pydatetime(), dom] = np.mean(date_df['Result']) 
    
            #Convert to date to actual date objects if resolution is day or greater
            if resolution != '15 min':
                df['Date'] = df['Date'].apply(lambda d: d.date())
        
            return df

        
        def parse_beta_values(days_cap=120):
            """
            """
            if self.beta_values_filepath is None: return None
            beta_val_list = pd.read_csv(self.beta_values_filepath)
            beta_val_list['Date'] = pd.to_datetime(beta_val_list['date'])
            subj_beta_vals = beta_val_list.loc[beta_val_list['id'] == self.id]
            if subj_beta_vals.empty: return subj_beta_vals[['Date', 'beta_a', 'beta_b']]
            first_date = subj_beta_vals.iloc[0]['Date']
            return subj_beta_vals.loc[(subj_beta_vals['Date'] >= first_date) & (subj_beta_vals['Date'] < first_date + datetime.timedelta(days=days_cap)), ['Date', 'beta_a', 'beta_b']]


        def parse_beiwe_pipeline(days_cap=120):
            """
            """
            #Find beiwe id
            if self.beiwe_filepath is None:
                return None, None 
            
            if not isinstance(self.beiwe_id, str):                
                return None, None 

            steps_file, sleep_file = None, None
            #Get step data
            if os.path.exists(os.path.join(self.beiwe_filepath, self.beiwe_id, 'steps.csv')):
                steps_file = pd.read_csv(os.path.join(self.beiwe_filepath, self.beiwe_id, 'steps.csv'))
                steps_file['Date'] = pd.to_datetime(steps_file['Date'], format='%Y-%m-%d')
                if steps_file.empty: pass
                else: steps_file = steps_file.loc[(steps_file['Date'] >= steps_file.iloc[0]['Date']) & (steps_file['Date'] < steps_file.iloc[0]['Date'] + datetime.timedelta(days=days_cap))]

            #Get sleep_data
            if os.path.exists(os.path.join(self.beiwe_filepath, self.beiwe_id, 'sleep_estimates.csv')):
                sleep_file = pd.read_csv(os.path.join(self.beiwe_filepath, self.beiwe_id, 'sleep_estimates.csv'))
                sleep_file['Date'] = pd.to_datetime(sleep_file['Date'], format='%Y-%m-%d')
                if sleep_file.empty: pass
                else: sleep_file = sleep_file.loc[(sleep_file['Date'] >= sleep_file.iloc[0]['Date']) & (sleep_file['Date'] < sleep_file.iloc[0]['Date'] + datetime.timedelta(days=days_cap))]

            return steps_file, sleep_file
 
        assert resolution in ['15 min', 'day', 'week', 'month']


        surveys = parse_surveys(days_cap=days_cap, 
                                day_first=day_first, 
                                day_last=day_last, 
                                resolution=resolution, 
                                start_monday=start_monday, 
                                start_morning=start_morning, 
                                time_centered=time_centered,
                                question_categories=question_categories)

        beta_vals = parse_beta_values(days_cap=days_cap)
        steps_file, sleep_file = parse_beiwe_pipeline(days_cap=days_cap)
        dataframes = [d for d in [surveys, beta_vals, steps_file, sleep_file] if d is not None] 
        if len(dataframes) == 0:
            df_final = pd.DataFrame({})
        else:
            df_final = reduce(lambda left,right: pd.merge(left, right, on='Date', how='outer'), dataframes).sort_values(by=['Date'])

        #Set domains
        #if self.domains is None: self.domains = [col for col in df_final.columns if col not in ['Date', 'id']]

        return df_final

    def impute(self, domains):
        """
        Get value for each column for each window
        """
        if self.impute_status:
            print('Dataframe already imputed.')
            return

        weighted_dict = [0.05, 0.20, 0.40, 1.5, 0.4, 0.20, 0.05]

        #Get indices of all middle bin values; add them to new df
        for dom in domains:
            if dom not in self.df:
                continue

            dom_values = []

            for ind in range(len(self.df.index)):

                #Get indices
                middle_weight_index = 3
                starting_index = max(ind -3, 0)
                ending_index = min(ind + 4, 90)

                #Get slice values
                subj_slice = self.df.iloc[starting_index:ending_index]

                #Remove na
                subj_slice_no_nan = subj_slice[dom].dropna()
                slice_indices = subj_slice_no_nan.index

                if len(slice_indices) == 0:
                    dom_values.append(np.nan)
                    continue

                #Match slice index with weight index
                weighted_dict_vals = [weighted_dict[middle_weight_index - (ind - slice_i)] for slice_i in slice_indices]

                #Find total in bin
                slice_val = sum(subj_slice_no_nan * [val / sum(weighted_dict_vals) for val in weighted_dict_vals])
                dom_values.append(slice_val)

            self.df[dom] = dom_values

        self.impute_status = True


    def bin(self, domains, window_size=3, shift=0):
        """
        Bin dataframe

        :param domains (list): the domains to bin 
        :window_size (int): the size of the bins (in days)
        :shift (int): the day of the week to start the binning on (Monday == 0)

        """

        domains = self.domain_check(domains)
        #Shift until Monday
        df_copy = self.df.copy()
        if shift is not None:
            try:
                dow = df_copy.iloc[0]['Date'].weekday()
                if dow > 0 and len(df_copy) > dow:
                    df_copy = df_copy.shift(shift - dow)
            except:
                print(self.id, df_copy)
        df_copy['bin'] = np.floor(df_copy.index / window_size )
        bins = df_copy.groupby('bin')
        subj_bin_df = pd.DataFrame(columns=['Bin Start Date', 'Bin End Date']+domains)
        for b in bins:
            bin_values = []
            #Add bin start/end dates
            
            start_date, end_date = b[1].iloc[0]['Date'], b[1].iloc[-1]['Date']
            bin_values.extend((start_date, end_date))
            for dom in domains:
                if dom in b[1].columns:
                    bin_dom_value = b[1][dom].mean()
                    bin_values.append(bin_dom_value)
                else:
                    bin_values.append(np.nan)

            #Add date range
            subj_bin_df.loc[b[0]] = bin_values    

        subj_bin_df['id'] = self.id
        
        self.bins = subj_bin_df
        
    def impute_bins(self, domains):
        """
        Try to impute bin objects
        """
        assert self.bins is not None
        
        for d in domains:
            for index, row in self.bins.iterrows():
                if 0 < index < len(self.bins.index) - 1:
                    index = int(index)
                    if pd.isnull(self.bins.iloc[index][d]) and not pd.isnull(self.bins.iloc[index-1][d]) and not pd.isnull(self.bins.iloc[index+1][d]):                        
                        self.bins.at[index,d] = np.mean([self.bins.iloc[index-1][d], self.bins.iloc[index+1][d]])


    def normalize(self, domains, domain_means={}, domain_vars={}):
        """
        Normalize columns values to 0 mean/ unit variance
        :param domain_means (dict): the mean for each column value
        :param domain_vars (dict): the variance for each column value

        If mean/var not provided, resort to in-sample normalization
        """
        if self.normalize_status:
            #print("Dataframe has already been normalized. Please reset dataframe if you wish to normalize it in a different way.")
            return
 
        domains = self.domain_check(domains)
        if domain_means == {} and domain_vars == {}:
            for dom in domains:
                if dom in self.df:
                    domain_means[dom] = self.df[dom].mean()
                    domain_vars[dom] = self.df[dom].std()

        for dom in domains:
            if dom in self.df.columns and dom in domain_means and dom in domain_vars:
                self.df[dom] = (self.df[dom] - domain_means[dom]) / domain_vars[dom]

            self.normalize_status = True

    def create_transition_dict(self, level):
        """
        Create nested dictionary structure 

        :param level (int): the level dictionary structure. Must be >= 0
        """
        trans_dict = {}
        for comb in itertools.product(('out', 'in'), repeat=level):
            trans_dict[comb] = {comb2:0 for comb2 in itertools.product(('out', 'in'), repeat=level)}
        return trans_dict


    def assign_transition_dict(self, trans_dict, row, row2):
        """
        Increment transition dict
        """
        label1 = tuple(['in' if col < 1.0 else 'out' for col in row])
        label2 = tuple(['in' if col < 1.0 else 'out' for col in row2])
        trans_dict[label1][label2] += 1

    def get_transitions(self, domains=None, joint_size=1):
        """
        Count transition events for each col in subj_df
        """
        domains = self.domain_check(domains)
        all_trans_dict = {}
        for dom_group in itertools.combinations(domains, r=joint_size):

            #Create trans dictionary
            group_dict = self.create_transition_dict(level=joint_size)

            #Find bins with values for each group
            good_bins = self.bins[list(dom_group)].dropna()

            #Assign
            row_iterator = good_bins.iterrows()
            try:
                last_i, last = next(row_iterator)
            except StopIteration:
                continue
            for index, row in row_iterator:
                if int(index) - int(last_i) <= 3:
                    self.assign_transition_dict(group_dict, last, row)
                last_i, last = index, row

            all_trans_dict[dom_group] = group_dict

        return all_trans_dict

    def domain_bouts(self, domains=None):
        """
        """
        def parse_bout_list(bout_list, state, low_bouts, high_bouts):
            """
            Helper function to parse bout list at end of bout
            """
            if len(bout_list) == 1: bout_list.append(bout_list[-1] + 3) #edge case where last domain event is only one in its bout

            if state: low_bouts.append(float(bout_list[-1]) - float(bout_list[0]))
            else: high_bouts.append(float(bout_list[-1]) - float(bout_list[0]))
            return low_bouts, high_bouts

        domains = self.domain_check(domains)
        bout_dict = {}
        for dom in domains:
            if dom not in self.df:
                continue

            bout_list = [] #temporary list that contains times of current bout
            subj_dom = self.df.loc[self.df[dom].notnull(), dom]
            row_iterator = subj_dom.iteritems()
            try:
                last_day, last_val = next(row_iterator)
                bout_list.append(last_day)
                if last_val < 1.0: last_state = True #set this back on first val
                else: last_state = False
            except StopIteration:
                continue

            bout_dict[dom] = {}
            low_bouts, high_bouts = [], [] #duration of all in-range bouts
            low_bouts_end, high_bouts_end = 0, 0 #counter the keep track of # of ended bout things
            for day, val in row_iterator:
                if val < 1.0: state = True
                else: state = False

                if last_state == state and day - last_day <= 6: #continue bout
                    bout_list.append(day)

                else: #discontinue bout
                    if day - last_day > 8: 
                        bout_list.append(last_day + 3) #If adjacent rows are day outside threshold, discontinue bout;cap last bout at 3 days past last activity	
                        if last_state: low_bouts_end += 1
                        else: high_bouts_end += 1
                    else: bout_list.append(day) #then normal switch

                    low_bouts, high_bouts = parse_bout_list(bout_list, last_state, low_bouts, high_bouts)
                    bout_list = [day]

                last_day, last_val = day, val
                last_state = state

            low_bouts, high_bouts = parse_bout_list(bout_list, last_state, low_bouts, high_bouts) #parse last bout
            bout_dict[dom]['low'], bout_dict[dom]['high'] = [float(b) for b in low_bouts], [float(b) for b in high_bouts]
            bout_dict[dom]['low ends'], bout_dict[dom]['high ends'] = low_bouts_end, high_bouts_end

        return bout_dict




from __future__ import absolute_import

from LAMP.api.api_api import APIApi
from LAMP.api.activity_api import ActivityApi
from LAMP.api.activity_event_api import ActivityEventApi
from LAMP.api.activity_spec_api import ActivitySpecApi
from LAMP.api.credential_api import CredentialApi
from LAMP.api.participant_api import ParticipantApi
from LAMP.api.researcher_api import ResearcherApi
from LAMP.api.sensor_api import SensorApi
from LAMP.api.sensor_event_api import SensorEventApi
from LAMP.api.sensor_spec_api import SensorSpecApi
from LAMP.api.study_api import StudyApi
from LAMP.api.type_api import TypeApi



# coding: utf-8

"""
    LAMP Platform

    The LAMP Platform API.

    The version of the OpenAPI document: 1.0.0
    Contact: team@digitalpsych.org
"""


# we can not import model classes here because that would create a circular
# reference which would not work in python2



# coding: utf-8

"""
    LAMP Platform

    The LAMP Platform API.

    The version of the OpenAPI document: 1.0.0
    Contact: team@digitalpsych.org
"""


from __future__ import absolute_import

import unittest

import LAMP
from LAMP.api.sensor_spec_api import SensorSpecApi
from LAMP.rest import ApiException


class TestSensorSpecApi(unittest.TestCase):
    """SensorSpecApi unit test stubs"""

    def setUp(self):
        self.api = LAMP.api.sensor_spec_api.SensorSpecApi()

    def tearDown(self):
        pass

    def test_all(self):
        """Test case for all

        Get all SensorSpecs registered.
        """
        pass

    def test_create(self):
        """Test case for create

        Create a new SensorSpec.
        """
        pass

    def test_delete(self):
        """Test case for delete

        Delete an SensorSpec.
        """
        pass

    def test_update(self):
        """Test case for update

        Update an SensorSpec.
        """
        pass

    def test_view(self):
        """Test case for view

        Get a SensorSpec.
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LAMP Platform

    The LAMP Platform API.

    The version of the OpenAPI document: 1.0.0
    Contact: team@digitalpsych.org
"""


from __future__ import absolute_import

import re
import sys

# python 2 and python 3 compatibility library
import six

from LAMP.api_client import ApiClient
from LAMP.exceptions import (
    ApiTypeError,
    ApiValueError
)
from LAMP.model_utils import (
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    int,
    none_type,
    str,
    validate_and_convert_types
)
from LAMP.models import error
from LAMP.models import dynamic_attachment


class TypeApi(object):
    """
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __get_attachment(
            self,
            type_id,
            attachment_key,
            **kwargs
        ):
            """get_attachment

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_attachment(type_id, attachment_key, async_req=True)
            >>> result = thread.get()

            Args:
                type_id (str):
                attachment_key (str):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bool, date, datetime, dict, float, int, list, str
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', False
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', False
            )
            kwargs['_host_index'] = kwargs.get('_host_index', 0)
            kwargs['type_id'] = \
                type_id
            kwargs['attachment_key'] = \
                attachment_key
            return self.call_with_http_info(**kwargs)

        self.get_attachment = Endpoint(
            settings={
                'response_type': (bool, date, datetime, dict, float, int, list, str,),
                'auth': [
                    'Authorization'
                ],
                'endpoint_path': '/type/{type_id}/attachment/{attachment_key}',
                'operation_id': 'get_attachment',
                'http_method': 'GET',
                'servers': [],
            },
            params_map={
                'all': [
                    'type_id',
                    'attachment_key',
                ],
                'required': [
                    'type_id',
                    'attachment_key',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'type_id':
                        (str,),
                    'attachment_key':
                        (str,),
                },
                'attribute_map': {
                    'type_id': 'type_id',
                    'attachment_key': 'attachment_key',
                },
                'location_map': {
                    'type_id': 'path',
                    'attachment_key': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__get_attachment
        )

        def __get_dynamic_attachment(
            self,
            type_id,
            attachment_key,
            invoke_always,
            include_logs,
            ignore_output,
            **kwargs
        ):
            """get_dynamic_attachment

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.get_dynamic_attachment(type_id, attachment_key, invoke_always, include_logs, ignore_output, async_req=True)
            >>> result = thread.get()

            Args:
                type_id (str):
                attachment_key (str):
                invoke_always (bool):
                include_logs (bool):
                ignore_output (bool):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bool, date, datetime, dict, float, int, list, str
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', False
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', False
            )
            kwargs['_host_index'] = kwargs.get('_host_index', 0)
            kwargs['type_id'] = \
                type_id
            kwargs['attachment_key'] = \
                attachment_key
            kwargs['invoke_always'] = \
                invoke_always
            kwargs['include_logs'] = \
                include_logs
            kwargs['ignore_output'] = \
                ignore_output
            return self.call_with_http_info(**kwargs)

        self.get_dynamic_attachment = Endpoint(
            settings={
                'response_type': (bool, date, datetime, dict, float, int, list, str,),
                'auth': [
                    'Authorization'
                ],
                'endpoint_path': '/type/{type_id}/attachment/dynamic/{attachment_key}',
                'operation_id': 'get_dynamic_attachment',
                'http_method': 'GET',
                'servers': [],
            },
            params_map={
                'all': [
                    'type_id',
                    'attachment_key',
                    'invoke_always',
                    'include_logs',
                    'ignore_output',
                ],
                'required': [
                    'type_id',
                    'attachment_key',
                    'invoke_always',
                    'include_logs',
                    'ignore_output',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'type_id':
                        (str,),
                    'attachment_key':
                        (str,),
                    'invoke_always':
                        (bool,),
                    'include_logs':
                        (bool,),
                    'ignore_output':
                        (bool,),
                },
                'attribute_map': {
                    'type_id': 'type_id',
                    'attachment_key': 'attachment_key',
                    'invoke_always': 'invoke_always',
                    'include_logs': 'include_logs',
                    'ignore_output': 'ignore_output',
                },
                'location_map': {
                    'type_id': 'path',
                    'attachment_key': 'path',
                    'invoke_always': 'query',
                    'include_logs': 'query',
                    'ignore_output': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__get_dynamic_attachment
        )

        def __list_attachments(
            self,
            type_id,
            **kwargs
        ):
            """list_attachments

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.list_attachments(type_id, async_req=True)
            >>> result = thread.get()

            Args:
                type_id (str):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bool, date, datetime, dict, float, int, list, str
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', False
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', False
            )
            kwargs['_host_index'] = kwargs.get('_host_index', 0)
            kwargs['type_id'] = \
                type_id
            return self.call_with_http_info(**kwargs)

        self.list_attachments = Endpoint(
            settings={
                'response_type': (bool, date, datetime, dict, float, int, list, str,),
                'auth': [
                    'Authorization'
                ],
                'endpoint_path': '/type/{type_id}/attachment',
                'operation_id': 'list_attachments',
                'http_method': 'GET',
                'servers': [],
            },
            params_map={
                'all': [
                    'type_id',
                ],
                'required': [
                    'type_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'type_id':
                        (str,),
                },
                'attribute_map': {
                    'type_id': 'type_id',
                },
                'location_map': {
                    'type_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__list_attachments
        )

        def __parent(
            self,
            type_id,
            **kwargs
        ):
            """Find the owner(s) of the resource.

            Get the parent type identifier of the data structure referenced by the identifier.
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.parent(type_id, async_req=True)
            >>> result = thread.get()

            Args:
                type_id (str):

            Keyword Args:
                transform (str): [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                str
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', False
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', False
            )
            kwargs['_host_index'] = kwargs.get('_host_index', 0)
            kwargs['type_id'] = \
                type_id
            return self.call_with_http_info(**kwargs)

        self.parent = Endpoint(
            settings={
                'response_type': (str,),
                'auth': [
                    'Authorization'
                ],
                'endpoint_path': '/type/{type_id}/parent',
                'operation_id': 'parent',
                'http_method': 'GET',
                'servers': [],
            },
            params_map={
                'all': [
                    'type_id',
                    'transform',
                ],
                'required': [
                    'type_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'type_id':
                        (str,),
                    'transform':
                        (str,),
                },
                'attribute_map': {
                    'type_id': 'type_id',
                    'transform': 'transform',
                },
                'location_map': {
                    'type_id': 'path',
                    'transform': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__parent
        )

        def __set_attachment(
            self,
            type_id,
            target,
            attachment_key,
            body,
            **kwargs
        ):
            """set_attachment

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.set_attachment(type_id, target, attachment_key, body, async_req=True)
            >>> result = thread.get()

            Args:
                type_id (str):
                target (str):
                attachment_key (str):
                body (bool, date, datetime, dict, float, int, list, str):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bool, date, datetime, dict, float, int, list, str
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', False
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', False
            )
            kwargs['_host_index'] = kwargs.get('_host_index', 0)
            kwargs['type_id'] = \
                type_id
            kwargs['target'] = \
                target
            kwargs['attachment_key'] = \
                attachment_key
            kwargs['body'] = \
                body
            return self.call_with_http_info(**kwargs)

        self.set_attachment = Endpoint(
            settings={
                'response_type': (bool, date, datetime, dict, float, int, list, str,),
                'auth': [
                    'Authorization'
                ],
                'endpoint_path': '/type/{type_id}/attachment/{attachment_key}/{target}',
                'operation_id': 'set_attachment',
                'http_method': 'PUT',
                'servers': [],
            },
            params_map={
                'all': [
                    'type_id',
                    'target',
                    'attachment_key',
                    'body',
                ],
                'required': [
                    'type_id',
                    'target',
                    'attachment_key',
                    'body',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'type_id':
                        (str,),
                    'target':
                        (str,),
                    'attachment_key':
                        (str,),
                    'body':
                        (bool, date, datetime, dict, float, int, list, str,),
                },
                'attribute_map': {
                    'type_id': 'type_id',
                    'target': 'target',
                    'attachment_key': 'attachment_key',
                },
                'location_map': {
                    'type_id': 'path',
                    'target': 'path',
                    'attachment_key': 'path',
                    'body': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__set_attachment
        )

        def __set_dynamic_attachment(
            self,
            type_id,
            target,
            attachment_key,
            invoke_once,
            dynamic_attachment_dynamic_attachment,
            **kwargs
        ):
            """set_dynamic_attachment

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True
            >>> thread = api.set_dynamic_attachment(type_id, target, attachment_key, invoke_once, dynamic_attachment_dynamic_attachment, async_req=True)
            >>> result = thread.get()

            Args:
                type_id (str):
                target (str):
                attachment_key (str):
                invoke_once (bool):
                dynamic_attachment_dynamic_attachment (dynamic_attachment.DynamicAttachment):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int): specifies the index of the server
                    that we want to use.
                    Default is 0.
                async_req (bool): execute request asynchronously

            Returns:
                bool, date, datetime, dict, float, int, list, str
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', False
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', False
            )
            kwargs['_host_index'] = kwargs.get('_host_index', 0)
            kwargs['type_id'] = \
                type_id
            kwargs['target'] = \
                target
            kwargs['attachment_key'] = \
                attachment_key
            kwargs['invoke_once'] = \
                invoke_once
            kwargs['dynamic_attachment_dynamic_attachment'] = \
                dynamic_attachment_dynamic_attachment
            return self.call_with_http_info(**kwargs)

        self.set_dynamic_attachment = Endpoint(
            settings={
                'response_type': (bool, date, datetime, dict, float, int, list, str,),
                'auth': [
                    'Authorization'
                ],
                'endpoint_path': '/type/{type_id}/attachment/dynamic/{attachment_key}/{target}',
                'operation_id': 'set_dynamic_attachment',
                'http_method': 'PUT',
                'servers': [],
            },
            params_map={
                'all': [
                    'type_id',
                    'target',
                    'attachment_key',
                    'invoke_once',
                    'dynamic_attachment_dynamic_attachment',
                ],
                'required': [
                    'type_id',
                    'target',
                    'attachment_key',
                    'invoke_once',
                    'dynamic_attachment_dynamic_attachment',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'type_id':
                        (str,),
                    'target':
                        (str,),
                    'attachment_key':
                        (str,),
                    'invoke_once':
                        (bool,),
                    'dynamic_attachment_dynamic_attachment':
                        (dynamic_attachment.DynamicAttachment,),
                },
                'attribute_map': {
                    'type_id': 'type_id',
                    'target': 'target',
                    'attachment_key': 'attachment_key',
                    'invoke_once': 'invoke_once',
                },
                'location_map': {
                    'type_id': 'path',
                    'target': 'path',
                    'attachment_key': 'path',
                    'invoke_once': 'query',
                    'dynamic_attachment_dynamic_attachment': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__set_dynamic_attachment
        )


class Endpoint(object):
    def __init__(self, settings=None, params_map=None, root_map=None,
                 headers_map=None, api_client=None, callable=None):
        """Creates an endpoint

        Args:
            settings (dict): see below key value pairs
                'response_type' (tuple/None): response type
                'auth' (list): a list of auth type keys
                'endpoint_path' (str): the endpoint path
                'operation_id' (str): endpoint string identifier
                'http_method' (str): POST/PUT/PATCH/GET etc
                'servers' (list): list of str servers that this endpoint is at
            params_map (dict): see below key value pairs
                'all' (list): list of str endpoint parameter names
                'required' (list): list of required parameter names
                'nullable' (list): list of nullable parameter names
                'enum' (list): list of parameters with enum values
                'validation' (list): list of parameters with validations
            root_map
                'validations' (dict): the dict mapping endpoint parameter tuple
                    paths to their validation dictionaries
                'allowed_values' (dict): the dict mapping endpoint parameter
                    tuple paths to their allowed_values (enum) dictionaries
                'openapi_types' (dict): param_name to openapi type
                'attribute_map' (dict): param_name to camelCase name
                'location_map' (dict): param_name to  'body', 'file', 'form',
                    'header', 'path', 'query'
                collection_format_map (dict): param_name to `csv` etc.
            headers_map (dict): see below key value pairs
                'accept' (list): list of Accept header strings
                'content_type' (list): list of Content-Type header strings
            api_client (ApiClient) api client instance
            callable (function): the function which is invoked when the
                Endpoint is called
        """
        self.settings = settings
        self.params_map = params_map
        self.params_map['all'].extend([
            'async_req',
            '_host_index',
            '_preload_content',
            '_request_timeout',
            '_return_http_data_only',
            '_check_input_type',
            '_check_return_type'
        ])
        self.params_map['nullable'].extend(['_request_timeout'])
        self.validations = root_map['validations']
        self.allowed_values = root_map['allowed_values']
        self.openapi_types = root_map['openapi_types']
        extra_types = {
            'async_req': (bool,),
            '_host_index': (int,),
            '_preload_content': (bool,),
            '_request_timeout': (none_type, int, (int,), [int]),
            '_return_http_data_only': (bool,),
            '_check_input_type': (bool,),
            '_check_return_type': (bool,)
        }
        self.openapi_types.update(extra_types)
        self.attribute_map = root_map['attribute_map']
        self.location_map = root_map['location_map']
        self.collection_format_map = root_map['collection_format_map']
        self.headers_map = headers_map
        self.api_client = api_client
        self.callable = callable

    def __validate_inputs(self, kwargs):
        for param in self.params_map['enum']:
            if param in kwargs:
                check_allowed_values(
                    self.allowed_values,
                    (param,),
                    kwargs[param]
                )

        for param in self.params_map['validation']:
            if param in kwargs:
                check_validations(
                    self.validations,
                    (param,),
                    kwargs[param]
                )

        if kwargs['_check_input_type'] is False:
            return

        for key, value in six.iteritems(kwargs):
            fixed_val = validate_and_convert_types(
                value,
                self.openapi_types[key],
                [key],
                False,
                kwargs['_check_input_type'],
                configuration=self.api_client.configuration
            )
            kwargs[key] = fixed_val

    def __gather_params(self, kwargs):
        params = {
            'body': None,
            'collection_format': {},
            'file': {},
            'form': [],
            'header': {},
            'path': {},
            'query': []
        }

        for param_name, param_value in six.iteritems(kwargs):
            param_location = self.location_map.get(param_name)
            if param_location is None:
                continue
            if param_location:
                if param_location == 'body':
                    params['body'] = param_value
                    continue
                base_name = self.attribute_map[param_name]
                if (param_location == 'form' and
                        self.openapi_types[param_name] == (file_type,)):
                    params['file'][param_name] = [param_value]
                elif (param_location == 'form' and
                        self.openapi_types[param_name] == ([file_type],)):
                    # param_value is already a list
                    params['file'][param_name] = param_value
                elif param_location in {'form', 'query'}:
                    param_value_full = (base_name, param_value)
                    params[param_location].append(param_value_full)
                if param_location not in {'form', 'query'}:
                    params[param_location][base_name] = param_value
                collection_format = self.collection_format_map.get(param_name)
                if collection_format:
                    params['collection_format'][base_name] = collection_format

        return params

    def __call__(self, *args, **kwargs):
        """ This method is invoked when endpoints are called
        Example:
        pet_api = PetApi()
        pet_api.add_pet  # this is an instance of the class Endpoint
        pet_api.add_pet()  # this invokes pet_api.add_pet.__call__()
        which then invokes the callable functions stored in that endpoint at
        pet_api.add_pet.callable or self.callable in this class
        """
        return self.callable(self, *args, **kwargs)

    def call_with_http_info(self, **kwargs):

        try:
            _host = self.settings['servers'][kwargs['_host_index']]
        except IndexError:
            if self.settings['servers']:
                raise ApiValueError(
                    "Invalid host index. Must be 0 <= index < %s" %
                    len(self.settings['servers'])
                )
            _host = None

        for key, value in six.iteritems(kwargs):
            if key not in self.params_map['all']:
                raise ApiTypeError(
                    "Got an unexpected parameter '%s'"
                    " to method `%s`" %
                    (key, self.settings['operation_id'])
                )
            # only throw this nullable ApiValueError if _check_input_type
            # is False, if _check_input_type==True we catch this case
            # in self.__validate_inputs
            if (key not in self.params_map['nullable'] and value is None
                    and kwargs['_check_input_type'] is False):
                raise ApiValueError(
                    "Value may not be None for non-nullable parameter `%s`"
                    " when calling `%s`" %
                    (key, self.settings['operation_id'])
                )

        for key in self.params_map['required']:
            if key not in kwargs.keys():
                raise ApiValueError(
                    "Missing the required parameter `%s` when calling "
                    "`%s`" % (key, self.settings['operation_id'])
                )

        self.__validate_inputs(kwargs)

        params = self.__gather_params(kwargs)

        accept_headers_list = self.headers_map['accept']
        if accept_headers_list:
            params['header']['Accept'] = self.api_client.select_header_accept(
                accept_headers_list)

        content_type_headers_list = self.headers_map['content_type']
        if content_type_headers_list:
            header_list = self.api_client.select_header_content_type(
                content_type_headers_list)
            params['header']['Content-Type'] = header_list

        return self.api_client.call_api(
            self.settings['endpoint_path'], self.settings['http_method'],
            params['path'],
            params['query'],
            params['header'],
            body=params['body'],
            post_params=params['form'],
            files=params['file'],
            response_type=self.settings['response_type'],
            auth_settings=self.settings['auth'],
            async_req=kwargs['async_req'],
            _check_type=kwargs['_check_return_type'],
            _return_http_data_only=kwargs['_return_http_data_only'],
            _preload_content=kwargs['_preload_content'],
            _request_timeout=kwargs['_request_timeout'],
            _host=_host,
            collection_formats=params['collection_format'])



# coding: utf-8

"""
    LAMP Platform

    The LAMP Platform API.

    The version of the OpenAPI document: 1.0.0
    Contact: team@digitalpsych.org
"""


from __future__ import absolute_import

import unittest

import LAMP


class TestSensorEvent(unittest.TestCase):
    """SensorEvent unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testSensorEvent(self):
        """Test SensorEvent"""
        # FIXME: construct object with mandatory attributes with example values
        # model = LAMP.SensorEvent()
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LAMP Platform

    The LAMP Platform API.

    The version of the OpenAPI document: 1.0.0
    Contact: team@digitalpsych.org
"""


from __future__ import absolute_import

import unittest

import LAMP
from LAMP.api.activity_api import ActivityApi
from LAMP.rest import ApiException


class TestActivityApi(unittest.TestCase):
    """ActivityApi unit test stubs"""

    def setUp(self):
        self.api = LAMP.api.activity_api.ActivityApi()

    def tearDown(self):
        pass

    def test_all(self):
        """Test case for all

        Get the set of all activities.
        """
        pass

    def test_all_by_participant(self):
        """Test case for all_by_participant

        Get all activities for a participant.
        """
        pass

    def test_all_by_researcher(self):
        """Test case for all_by_researcher

        Get all activities for a researcher.
        """
        pass

    def test_all_by_study(self):
        """Test case for all_by_study

        Get all activities in a study.
        """
        pass

    def test_create(self):
        """Test case for create

        Create a new Activity under the given Study.
        """
        pass

    def test_delete(self):
        """Test case for delete

        Delete an Activity.
        """
        pass

    def test_update(self):
        """Test case for update

        Update an Activity's settings.
        """
        pass

    def test_view(self):
        """Test case for view

        Get a single activity, by identifier.
        """
        pass


if __name__ == '__main__':
    unittest.main()



import LAMP
import numpy as np
from functools import reduce

class StudyExt():
    """
    """
    def __init__(self, 
                 subjects, 
                 domains=None, 
                 beiwe_filepath=None, 
                 beta_values_filepath=None, 
                 ages=None, 
                 races=None, 
                 sexes=None, 
                 beiwe_ids=None,
                 df_props={}):

        self.beiwe_filepath = beiwe_filepath
        self.beta_values_filepath = beta_values_filepath
        self.ages = ages
        self.races = races
        self.sexes = sexes
        self.beiwe_ids = beiwe_ids
        self.domains = domains
        self.df_props = df_props
        self.init_subjects(subjects)

    def __iter__(self):
        for subj in self.subjects:
            yield subj

    def __len__(self):
        return len(self.subjects)

    def __getitem__(self, key):
        return self.subjects[key]

    @property
    def subjects(self):
        return self._subjects
    @property
    def beiwe_filepath(self):
        return self._beiwe_filepath
    
    @property
    def beta_values_filepath(self):
        return self._beta_values_filepath

    @property
    def ages(self):
        return self._ages

    @property
    def races(self):
        return self._races

    @property
    def sexes(self):
        return self._sexes

    @property
    def beiwe_ids(self):
        return self._beiwe_ids

    @property
    def domains(self):
        return self._domains

    @subjects.setter
    def subjects(self, value):
        self._subjects = value

    @beiwe_filepath.setter
    def beiwe_filepath(self, value):
        self._beiwe_filepath = value
        
    @beta_values_filepath.setter
    def beta_values_filepath(self, value):
        self._beta_values_filepath = value

    @ages.setter
    def ages(self, value):
        self._ages = value

    @races.setter	
    def races(self, value):
        self._races = value

    @sexes.setter	
    def sexes(self, value):
        self._sexes = value

    @beiwe_ids.setter
    def beiwe_ids(self, value):
        self._beiwe_ids = value

    @domains.setter
    def domains(self, value):
        self._domains = value

#     def select(self, subjects):
#         """
#         """
    def domain_check(self, domains):
        """
        If domains is passed in, just return it.

        Else, see if domains is set as object attribute
        """
        if domains is None:
            if not hasattr(self, 'domains'):
                raise AttributeError('Domains were not set for cohort and were not provided.')
            domains = self.domains
        return domains

    def init_subjects(self, subjects):
        """
        Initialize subjects in cohorts. Can take either Subject objects or subject ids
        """
        self.subjects = []
        for subj in subjects:
            print(subj)
            if isinstance(subj, LAMP.analysis.ParticipantExt):
                #NEED TO CHECK SUBJECT MATCHES PROPS
                self.subjects.append(subj)

            else:
                self.add_subject(subj)

    def get_subject(self, subj_id):
        """
        Get Subject object by id.

        If it doesn't exist, will return None
        """
        for subj in self.subjects:
            if subj.id == subj_id:
                return subj

        print('Subject not found!')
        return None

    def add_subject(self, subject):
        subject_age = self.ages[subject] if self.ages and subject in self.ages else None
        subject_race = self.races[subject] if self.races and subject in self.races else None
        subject_sex = self.sexes[subject] if self.sexes and subject in self.sexes else None
        subject_beiwe_id = self.beiwe_ids[subject] if self.beiwe_ids and subject in self.beiwe_ids else None
        self.subjects.append(LAMP.analysis.Participant(id = subject, 
                                          beiwe_filepath = self.beiwe_filepath, 
                                          beta_values_filepath = self.beta_values_filepath, 
                                          domains=self.domains, 
                                          age=subject_age, 
                                          race=subject_race, 
                                          sex=subject_sex, 
                                          beiwe_id=subject_beiwe_id,
                                          df_props=self.df_props))


    def mean_age(self):
        """
        Return mean, std age of subjects in cohort
        """
        subject_ages = [subj.age for subj in self.subjects if subj.age is not None]
        if len(subject_ages) == 0:
            return None

        return np.mean(subject_ages)


    def std_age(self):
        """
        Return std age of subjects in cohort
        """
        subject_ages = [subj.age for subj in self.subjects if subj.age is not None]
        if len(subject_ages) == 0:
            return None

        return np.std(subject_ages)

    def set_ages(self, subject_ages):
        """
        Set age for existing subject in cohort.

        If subject in dictionary cannot be found, id will be ignored

        :param subject_ages (dict): maps subject id to age
        """

        for subj in self.subjects:
            if subj.id in subject_ages:
                subj.age = subject_ages[subj.id]


    def set_races(self, subject_races):
        """
        Set race for existing subject in cohort.

        If subject in dictionary cannot be found, id will be ignored

        :param subject_races (dict): maps subject id to race
        """

        for subj in self.subjects:
            if subj.id in subject_races:
                subj.race = subject_races[subj.id]


    def set_sex(self, subject_sex):
        """
        Set sex for existing subject in cohort.

        If subject in dictionary cannot be found, id will be ignored

        :param subject_sex (dict): maps subject id to sex
        """

        for subj in self.subjects:
            if subj.id in subject_sex:
                subj.sex = subject_sex[subj.id]

    def set_beiwe_ids(self, subject_beiwe_ids):
        """
        Set beiwe_ids for existing subjects in cohort.

        If subject in dictionary cannot be found, id will be ignored

        :param subject_beiwe_ids (dict): maps subject id to beiwe_id
        """

        for subj in self.subjects:
            if subj.id in subject_beiwe_ids:
                subj.beiwe_id = subject_beiwe_ids[subj.id]


    def domain_mean(self, domain):
        """
        Find the mean value for particular domain in cohort
        """
        dom_values = [subj.df[domain].values for subj in self.subjects if domain in subj.df.columns]
        if len(dom_values) == 0: return None
        return np.nanmean(np.concatenate(dom_values))

    def domain_stdev(self, domain):
        """
        Find the std value for particular domain in cohort
        """
        dom_values = [subj.df[domain].values for subj in self.subjects if domain in subj.df.columns]
        if len(dom_values) == 0: return None
        return np.nanstd(np.concatenate(dom_values))

    def normalize(self, domains=None, dom_means=None, dom_vars=None, in_sample=False):
        """
        Normalize each domain in cohort so that values have 0 mean/unit variance

        If in_sample is true, then performs within-sample normalization
        """
        if domains is None:
            if not hasattr(self, 'domains'):
                raise AttributeError('Domains were not set for cohort and were not provided.')
            domains = self.domains

        if in_sample:
            for subj in self.subjects: subj.normalize(domains)
        else:
            if dom_means is None:
                dom_means = {dom: self.domain_mean(dom) for dom in domains if self.domain_mean(dom) is not None}
            if dom_vars is None:
                dom_vars = {dom: self.domain_stdev(dom) for dom in domains if self.domain_stdev(dom) is not None}
            for subj in self.subjects: subj.normalize(domains=domains, domain_means=dom_means, domain_vars=dom_vars)

    def impute(self, domains=None):
        """
        Impute every subject in cohort.
        """
        if domains is None:
            if not hasattr(self, 'domains'):
                raise AttributeError('Domains were not set for cohort and were not provided.')
            domains = self.domains

        for subj in self: subj.impute(domains=domains)

    def bin(self, domains=None, window_size=3):
        """
        Bins all subjects in cohort.
        """
        if domains is None:
            if not hasattr(self, 'domains'):
                raise AttributeError('Domains were not set for cohort and were not provided.')
            domains = self.domains

        for subj in self: subj.bin(domains=domains, window_size=window_size)
            
    def impute_bins(self, domains=None):
        """
        Impute bins
        """
        if domains is None:
            if not hasattr(self, 'domains'):
                raise AttributeError('Domains were not set for cohort and were not provided.')
            domains = self.domains
            
        for subj in self: subj.impute_bins(domains=domains)
        

    def transition_probabilities(self, domains=None, joint_size=1):
        """
        Get cohort_wide transistion probabilities.

        :param joint_size (int): the number of variables used when calculating the joint probabilities for transistion event. Defaults to 1. 
        """
        if domains is None:
            if not hasattr(self, 'domains'):
                raise AttributeError('Domains were not set for cohort and were not provided.')
            domains = self.domains

        samples_tp = [pro.get_transitions(domains = domains, joint_size = joint_size) for pro in self]

        master_dict = {}

        for sample in samples_tp:
            for cat in sample:				
                if cat not in master_dict:
                    master_dict[cat] = {state: sample[cat][state] for state in sample[cat]}
                else: #merge
                    for state in sample[cat]:
                        master_dict[cat][state] = {state2: master_dict[cat][state][state2] + sample[cat][state][state2] for state2 in 
                                                                                                                        master_dict[cat][state]}			
        #Convert to probabilities
        trans_dict = {}
        for cat in master_dict:
            trans_dict[cat] = {}
            for state in master_dict[cat]:
                trans_dict[cat][state] = {}
                for state2 in master_dict[cat][state]:
                    if sum(master_dict[cat][state].values()) == 0:
                        trans_dict[cat][state][state2] = None
                    else:
                        trans_dict[cat][state][state2] = float(master_dict[cat][state][state2]) / float(sum(master_dict[cat][state].values()))			
        return trans_dict, master_dict

    def domain_bouts(self, domains=None):
        if domains is None:
            if not hasattr(self, 'domains'):
                raise AttributeError('Domains were not set for cohort and were not provided.')
            domains = self.domains

        bout_dict = {}
        for subj in self: 
            subj_bout_dict = subj.domain_bouts(domains=domains)
            for dom in subj_bout_dict:
                if dom not in bout_dict: bout_dict[dom] = subj_bout_dict[dom]
                else: 
                    bout_dict[dom]['low'] += subj_bout_dict[dom]['low']
                    bout_dict[dom]['high'] += subj_bout_dict[dom]['high']
                    bout_dict[dom]['low ends'] += subj_bout_dict[dom]['low ends']
                    bout_dict[dom]['high ends'] += subj_bout_dict[dom]['high ends']

        return bout_dict












# coding: utf-8

"""
    LAMP Platform

    The LAMP Platform API.

    The version of the OpenAPI document: 1.0.0
    Contact: team@digitalpsych.org
"""


from __future__ import absolute_import

import unittest

import LAMP
from LAMP.api.activity_event_api import ActivityEventApi
from LAMP.rest import ApiException


class TestActivityEventApi(unittest.TestCase):
    """ActivityEventApi unit test stubs"""

    def setUp(self):
        self.api = LAMP.api.activity_event_api.ActivityEventApi()

    def tearDown(self):
        pass

    def test_all_by_participant(self):
        """Test case for all_by_participant

        Get all activity events for a participant.
        """
        pass

    def test_all_by_researcher(self):
        """Test case for all_by_researcher

        Get all activity events for a researcher by participant.
        """
        pass

    def test_all_by_study(self):
        """Test case for all_by_study

        Get all activity events for a study by participant.
        """
        pass

    def test_create(self):
        """Test case for create

        Create a new ActivityEvent for the given Participant.
        """
        pass

    def test_delete(self):
        """Test case for delete

        Delete a ActivityEvent.
        """
        pass


if __name__ == '__main__':
    unittest.main()
